@@ -11,8 +11,6 @@
 
 #include "adw-animation-util.h"
 #include "adw-easing.h"
-#include "adw-enums.h"
-#include "adw-length-unit.h"
 
 /**
  * AdwClampLayout:
@@ -36,10 +34,9 @@
  * If a child requires more than the requested maximum size, it will be
  * allocated the minimum size it can fit in instead.
  *
- * `AdwClampLayout` can scale with the text scale factor, use the
- * [property@ClampLayout:unit] property to enable that behavior.
- *
- * See also: [class@Clamp], [class@ClampScrollable].
+ * Each child will get the style  classes .large when it reached its maximum
+ * size, .small when it's allocated the full size, .medium in-between, or none
+ * if it hasn't been allocated yet.
  */
 
 #define ADW_EASE_OUT_TAN_CUBIC 3
@@ -48,12 +45,11 @@ enum {
   PROP_0,
   PROP_MAXIMUM_SIZE,
   PROP_TIGHTENING_THRESHOLD,
-  PROP_UNIT,
 
   /* Overridden properties */
   PROP_ORIENTATION,
 
-  LAST_PROP = PROP_UNIT + 1,
+  LAST_PROP = PROP_TIGHTENING_THRESHOLD + 1,
 };
 
 struct _AdwClampLayout
@@ -62,7 +58,6 @@ struct _AdwClampLayout
 
   int maximum_size;
   int tightening_threshold;
-  AdwLengthUnit unit;
 
   GtkOrientation orientation;
 };
@@ -101,9 +96,6 @@ adw_clamp_layout_get_property (GObject    *object,
   case PROP_TIGHTENING_THRESHOLD:
     g_value_set_int (value, adw_clamp_layout_get_tightening_threshold (self));
     break;
-  case PROP_UNIT:
-    g_value_set_enum (value, adw_clamp_layout_get_unit (self));
-    break;
   case PROP_ORIENTATION:
     g_value_set_enum (value, self->orientation);
     break;
@@ -127,9 +119,6 @@ adw_clamp_layout_set_property (GObject      *object,
   case PROP_TIGHTENING_THRESHOLD:
     adw_clamp_layout_set_tightening_threshold (self, g_value_get_int (value));
     break;
-  case PROP_UNIT:
-    adw_clamp_layout_set_unit (self, g_value_get_enum (value));
-    break;
   case PROP_ORIENTATION:
     set_orientation (self, g_value_get_enum (value));
     break;
@@ -138,52 +127,32 @@ adw_clamp_layout_set_property (GObject      *object,
   }
 }
 
-/* Given adw_lerp (a, b, t) -> r, find t. In other words: we know what sizes
- * we're interpolating between and the current size; find the current
- * interpolation progress.
- *
- * Note that this is different from the similarly named inverse_lerp function
- * in adw-view-stack.c. That one finds b given a, r, and t, while this one
- * finds t given a, b, and r.
- */
 static inline double
 inverse_lerp (double a,
               double b,
-              double r)
+              double t)
 {
-  return (r - a) / (b - a);
+  return (t - a) / (b - a);
 }
 
 static int
 clamp_size_from_child (AdwClampLayout *self,
-                       GtkSettings    *settings,
-                       GtkWidget      *child,
-                       int             child_size)
+                       int             min,
+                       int             nat)
 {
-  double max, lower, upper, progress, maximum_size, tightening_threshold;
-  int min, nat;
+  int max = 0, lower = 0, upper = 0;
+  double progress;
 
-  maximum_size = adw_length_unit_to_px (self->unit,
-                                        self->maximum_size,
-                                        settings);
-  tightening_threshold = adw_length_unit_to_px (self->unit,
-                                                self->tightening_threshold,
-                                                settings);
-
-  gtk_widget_measure (child, self->orientation, -1, &min, &nat, NULL, NULL);
-
-  lower = MAX (MIN (tightening_threshold, maximum_size), min);
-  max = MAX (lower, maximum_size);
+  lower = MAX (MIN (self->tightening_threshold, self->maximum_size), min);
+  max = MAX (lower, self->maximum_size);
   upper = lower + ADW_EASE_OUT_TAN_CUBIC * (max - lower);
 
-  g_assert (child_size != -1);
-
-  if (child_size <= lower)
-    return child_size;
-  else if (child_size >= max)
-    return upper;
+  if (nat <= lower)
+    progress = 0;
+  else if (nat >= max)
+    progress = 1;
   else {
-    double ease = inverse_lerp (lower, max, child_size);
+    double ease = inverse_lerp (lower, max, nat);
 
     progress = 1 + cbrt (ease - 1); // inverse ease out cubic
   }
@@ -193,45 +162,48 @@ clamp_size_from_child (AdwClampLayout *self,
 
 static int
 child_size_from_clamp (AdwClampLayout *self,
-                       GtkSettings    *settings,
                        GtkWidget      *child,
                        int             for_size,
                        int            *child_maximum,
                        int            *lower_threshold)
 {
-  double lower, upper, max, progress, maximum_size, tightening_threshold;
-  int min = 0, nat = 0;
-
-  maximum_size = adw_length_unit_to_px (self->unit,
-                                        self->maximum_size,
-                                        settings);
-  tightening_threshold = adw_length_unit_to_px (self->unit,
-                                                self->tightening_threshold,
-                                                settings);
+  int min = 0, nat = 0, max = 0, lower = 0, upper = 0;
+  double progress;
 
   gtk_widget_measure (child, self->orientation, -1, &min, &nat, NULL, NULL);
 
-  lower = MAX (MIN (tightening_threshold, maximum_size), min);
-  max = MAX (lower, maximum_size);
+  lower = MAX (MIN (self->tightening_threshold, self->maximum_size), min);
+  max = MAX (lower, self->maximum_size);
   upper = lower + ADW_EASE_OUT_TAN_CUBIC * (max - lower);
 
   if (child_maximum)
-    *child_maximum = ceil (max);
+    *child_maximum = max;
   if (lower_threshold)
-    *lower_threshold = ceil (lower);
+    *lower_threshold = lower;
 
   if (for_size < 0)
-    return MIN (nat, ceil (max));
+    return MIN (nat, max);
 
   if (for_size <= lower)
     return for_size;
 
   if (for_size >= upper)
-    return ceil (max);
+    return max;
 
   progress = inverse_lerp (lower, upper, for_size);
 
-  return floor (adw_lerp (lower, max, adw_easing_ease (ADW_EASE_OUT_CUBIC, progress)));
+  return adw_lerp (lower, max, adw_easing_ease (ADW_EASE_OUT_CUBIC, progress));
+}
+
+static GtkSizeRequestMode
+adw_clamp_layout_get_request_mode (GtkLayoutManager *manager,
+                                   GtkWidget        *widget)
+{
+  AdwClampLayout *self = ADW_CLAMP_LAYOUT (manager);
+
+  return self->orientation == GTK_ORIENTATION_HORIZONTAL ?
+    GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH :
+    GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT;
 }
 
 static void
@@ -245,73 +217,40 @@ adw_clamp_layout_measure (GtkLayoutManager *manager,
                           int              *natural_baseline)
 {
   AdwClampLayout *self = ADW_CLAMP_LAYOUT (manager);
-  GtkSettings *settings = gtk_widget_get_settings (widget);
   GtkWidget *child;
 
   for (child = gtk_widget_get_first_child (widget);
        child != NULL;
        child = gtk_widget_get_next_sibling (child)) {
-    int clamp_min = 0;
-    int clamp_nat = 0;
     int child_min = 0;
     int child_nat = 0;
-    int min_baseline = -1;
-    int nat_baseline = -1;
+    int child_min_baseline = -1;
+    int child_nat_baseline = -1;
 
     if (!gtk_widget_should_layout (child))
       continue;
 
     if (self->orientation == orientation) {
-      /* We pass all of for_size to the child, but we have to post-process
-       * its measurements.
-       */
       gtk_widget_measure (child, orientation, for_size,
                           &child_min, &child_nat,
-                          &min_baseline, &nat_baseline);
+                          &child_min_baseline, &child_nat_baseline);
 
-      clamp_min = clamp_size_from_child (self, settings, child, child_min);
-      clamp_nat = clamp_size_from_child (self, settings, child, child_nat);
-      if (min_baseline > -1)
-        min_baseline += (clamp_min - child_min) / 2;
-      if (nat_baseline > -1)
-        nat_baseline += (clamp_nat - child_nat) / 2;
+      child_nat = clamp_size_from_child (self, child_min, child_nat);
     } else {
-      /* We clamp for_size, but after that, child's minimum and natural
-       * sizes in this orientation are what we report.
-       */
-      int child_maximum = 0;
-      int child_size = child_size_from_clamp (self, settings, child,
-                                              for_size, &child_maximum,
-                                              NULL);
+      int child_size = child_size_from_clamp (self, child, for_size, NULL, NULL);
 
       gtk_widget_measure (child, orientation, child_size,
                           &child_min, &child_nat,
-                          &min_baseline, &nat_baseline);
-
-      if (for_size == -1 && child_size < child_maximum) {
-        /* For any specific for_size, we have a definite child_size that
-         * we measure the child for. When passed for_size = -1 however,
-         * we're supposed to report overall minimum size, for any
-         * potential value of for_size. For that, we have to measure the
-         * child at the maximum size we could give it. Note that we
-         * should not use this measurement for the natural size.
-         */
-        gtk_widget_measure (child, orientation, child_maximum,
-                            &child_min, NULL,
-                            &min_baseline, NULL);
-      }
-
-      clamp_min = child_min;
-      clamp_nat = child_nat;
+                          &child_min_baseline, &child_nat_baseline);
     }
 
-    *minimum = MAX (*minimum, clamp_min);
-    *natural = MAX (*natural, clamp_nat);
+    *minimum = MAX (*minimum, child_min);
+    *natural = MAX (*natural, child_nat);
 
-    if (min_baseline > -1)
-      *minimum_baseline = MAX (*minimum_baseline, min_baseline);
-    if (nat_baseline > -1)
-      *natural_baseline = MAX (*natural_baseline, nat_baseline);
+    if (child_min_baseline > -1)
+      *minimum_baseline = MAX (*minimum_baseline, child_min_baseline);
+    if (child_nat_baseline > -1)
+      *natural_baseline = MAX (*natural_baseline, child_nat_baseline);
   }
 }
 
@@ -323,7 +262,6 @@ adw_clamp_layout_allocate (GtkLayoutManager *manager,
                            int               baseline)
 {
   AdwClampLayout *self = ADW_CLAMP_LAYOUT (manager);
-  GtkSettings *settings = gtk_widget_get_settings (widget);
   GtkWidget *child;
 
   for (child = gtk_widget_get_first_child (widget);
@@ -338,12 +276,11 @@ adw_clamp_layout_allocate (GtkLayoutManager *manager,
       gtk_widget_remove_css_class (child, "medium");
       gtk_widget_remove_css_class (child, "large");
 
-      continue;
+      return;
     }
 
     if (self->orientation == GTK_ORIENTATION_HORIZONTAL) {
-      child_allocation.width = child_size_from_clamp (self, settings,
-                                                      child, width,
+      child_allocation.width = child_size_from_clamp (self, child, width,
                                                       &child_maximum,
                                                       &lower_threshold);
       child_allocation.height = height;
@@ -351,8 +288,7 @@ adw_clamp_layout_allocate (GtkLayoutManager *manager,
       child_clamped_size = child_allocation.width;
     } else {
       child_allocation.width = width;
-      child_allocation.height = child_size_from_clamp (self, settings,
-                                                       child, height,
+      child_allocation.height = child_size_from_clamp (self, child, height,
                                                        &child_maximum,
                                                        &lower_threshold);
 
@@ -395,6 +331,7 @@ adw_clamp_layout_class_init (AdwClampLayoutClass *klass)
   object_class->get_property = adw_clamp_layout_get_property;
   object_class->set_property = adw_clamp_layout_set_property;
 
+  layout_manager_class->get_request_mode = adw_clamp_layout_get_request_mode;
   layout_manager_class->measure = adw_clamp_layout_measure;
   layout_manager_class->allocate = adw_clamp_layout_allocate;
 
@@ -403,7 +340,7 @@ adw_clamp_layout_class_init (AdwClampLayoutClass *klass)
                                     "orientation");
 
   /**
-   * AdwClampLayout:maximum-size:
+   * AdwClampLayout:maximum-size: (attributes org.gtk.Property.get=adw_clamp_layout_get_maximum_size org.gtk.Property.set=adw_clamp_layout_set_maximum_size)
    *
    * The maximum size to allocate to the children.
    *
@@ -416,7 +353,7 @@ adw_clamp_layout_class_init (AdwClampLayoutClass *klass)
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwClampLayout:tightening-threshold:
+   * AdwClampLayout:tightening-threshold: (attributes org.gtk.Property.get=adw_clamp_layout_get_tightening_threshold org.gtk.Property.set=adw_clamp_layout_set_tightening_threshold)
    *
    * The size above which the children are clamped.
    *
@@ -438,21 +375,6 @@ adw_clamp_layout_class_init (AdwClampLayoutClass *klass)
                       0, G_MAXINT, 400,
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
-  /**
-   * AdwClampLayout:unit:
-   *
-   * The length unit for maximum size and tightening threshold.
-   *
-   * Allows the sizes to vary depending on the text scale factor.
-   *
-   * Since: 1.4
-   */
-  props[PROP_UNIT] =
-    g_param_spec_enum ("unit", NULL, NULL,
-                       ADW_TYPE_LENGTH_UNIT,
-                       ADW_LENGTH_UNIT_SP,
-                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
   g_object_class_install_properties (object_class, LAST_PROP, props);
 }
 
@@ -461,7 +383,6 @@ adw_clamp_layout_init (AdwClampLayout *self)
 {
   self->maximum_size = 600;
   self->tightening_threshold = 400;
-  self->unit = ADW_LENGTH_UNIT_SP;
 }
 
 /**
@@ -478,7 +399,7 @@ adw_clamp_layout_new (void)
 }
 
 /**
- * adw_clamp_layout_get_maximum_size:
+ * adw_clamp_layout_get_maximum_size: (attributes org.gtk.Method.get_property=maximum-size)
  * @self: a clamp layout
  *
  * Gets the maximum size allocated to the children.
@@ -494,7 +415,7 @@ adw_clamp_layout_get_maximum_size (AdwClampLayout *self)
 }
 
 /**
- * adw_clamp_layout_set_maximum_size:
+ * adw_clamp_layout_set_maximum_size: (attributes org.gtk.Method.set_property=maximum-size)
  * @self: a clamp layout
  * @maximum_size: the maximum size
  *
@@ -519,7 +440,7 @@ adw_clamp_layout_set_maximum_size (AdwClampLayout *self,
 }
 
 /**
- * adw_clamp_layout_get_tightening_threshold:
+ * adw_clamp_layout_get_tightening_threshold: (attributes org.gtk.Method.get_property=tightening-threshold)
  * @self: a clamp layout
  *
  * Gets the size above which the children are clamped.
@@ -535,7 +456,7 @@ adw_clamp_layout_get_tightening_threshold (AdwClampLayout *self)
 }
 
 /**
- * adw_clamp_layout_set_tightening_threshold:
+ * adw_clamp_layout_set_tightening_threshold: (attributes org.gtk.Method.set_property=tightening-threshold)
  * @self: a clamp layout
  * @tightening_threshold: the tightening threshold
  *
@@ -569,50 +490,3 @@ adw_clamp_layout_set_tightening_threshold (AdwClampLayout *self,
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_TIGHTENING_THRESHOLD]);
 }
-
-/**
- * adw_clamp_layout_get_unit:
- * @self: a clamp layout
- *
- * Gets the length unit for maximum size and tightening threshold.
- *
- * Returns: the length unit
- *
- * Since: 1.4
- */
-AdwLengthUnit
-adw_clamp_layout_get_unit (AdwClampLayout *self)
-{
-  g_return_val_if_fail (ADW_IS_CLAMP_LAYOUT (self), ADW_LENGTH_UNIT_PX);
-
-  return self->unit;
-}
-
-/**
- * adw_clamp_layout_set_unit:
- * @self: a clamp layout
- * @unit: the length unit
- *
- * Sets the length unit for maximum size and tightening threshold.
- *
- * Allows the sizes to vary depending on the text scale factor.
- *
- * Since: 1.4
- */
-void
-adw_clamp_layout_set_unit (AdwClampLayout *self,
-                           AdwLengthUnit   unit)
-{
-  g_return_if_fail (ADW_IS_CLAMP_LAYOUT (self));
-  g_return_if_fail (unit >= ADW_LENGTH_UNIT_PX);
-  g_return_if_fail (unit <= ADW_LENGTH_UNIT_SP);
-
-  if (unit == self->unit)
-    return;
-
-  self->unit = unit;
-
-  gtk_layout_manager_layout_changed (GTK_LAYOUT_MANAGER (self));
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_UNIT]);
-}
