@@ -25,14 +25,9 @@
  * which in turn are represented by [class@PreferencesRow].
  *
  * To summarize the role of the preferences it gathers, a group can have both a
- * title and a description. The title will be used by [class@PreferencesDialog]
+ * title and a description. The title will be used by [class@PreferencesWindow]
  * to let the user look for a preference.
  *
- * The [property@PreferencesGroup:separate-rows] property can be used to
- * separate the rows within the group, same as when using the
- * [`.boxed-list-separate`](style-classes.html#boxed-lists-cards) style class
- * instead of `.boxed-list`.
- *
  * ## AdwPreferencesGroup as GtkBuildable
  *
  * The `AdwPreferencesGroup` implementation of the [iface@Gtk.Buildable] interface
@@ -79,7 +74,6 @@ enum {
   PROP_TITLE,
   PROP_DESCRIPTION,
   PROP_HEADER_SUFFIX,
-  PROP_SEPARATE_ROWS,
   LAST_PROP,
 };
 
@@ -174,8 +168,7 @@ row_has_title (AdwPreferencesRow *row,
 {
   const char *title;
 
-  if (!ADW_IS_PREFERENCES_ROW (row))
-    return false;
+  g_assert (ADW_IS_PREFERENCES_ROW (row));
 
   if (!gtk_widget_get_visible (GTK_WIDGET (row)))
     return FALSE;
@@ -203,9 +196,6 @@ adw_preferences_group_get_property (GObject    *object,
   case PROP_HEADER_SUFFIX:
     g_value_set_object (value, adw_preferences_group_get_header_suffix (self));
     break;
-  case PROP_SEPARATE_ROWS:
-    g_value_set_boolean (value, adw_preferences_group_get_separate_rows (self));
-    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -229,9 +219,6 @@ adw_preferences_group_set_property (GObject      *object,
   case PROP_HEADER_SUFFIX:
     adw_preferences_group_set_header_suffix (self, g_value_get_object (value));
     break;
-  case PROP_SEPARATE_ROWS:
-    adw_preferences_group_set_separate_rows (self, g_value_get_boolean (value));
-    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -261,10 +248,9 @@ adw_preferences_group_class_init (AdwPreferencesGroupClass *klass)
   object_class->dispose = adw_preferences_group_dispose;
 
   widget_class->compute_expand = adw_widget_compute_expand;
-  widget_class->focus = adw_widget_focus_child;
 
   /**
-   * AdwPreferencesGroup:title:
+   * AdwPreferencesGroup:title: (attributes org.gtk.Property.get=adw_preferences_group_get_title org.gtk.Property.set=adw_preferences_group_set_title)
    *
    * The title for this group of preferences.
    */
@@ -274,7 +260,7 @@ adw_preferences_group_class_init (AdwPreferencesGroupClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwPreferencesGroup:description:
+   * AdwPreferencesGroup:description: (attributes org.gtk.Property.get=adw_preferences_group_get_description org.gtk.Property.set=adw_preferences_group_set_description)
    *
    * The description for this group of preferences.
    */
@@ -284,7 +270,7 @@ adw_preferences_group_class_init (AdwPreferencesGroupClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwPreferencesGroup:header-suffix:
+   * AdwPreferencesGroup:header-suffix: (attributes org.gtk.Property.get=adw_preferences_group_get_header_suffix org.gtk.Property.set=adw_preferences_group_set_header_suffix)
    *
    * The header suffix widget.
    *
@@ -300,22 +286,6 @@ adw_preferences_group_class_init (AdwPreferencesGroupClass *klass)
                          GTK_TYPE_WIDGET,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
-  /**
-   * AdwPreferencesGroup:separate-rows:
-   *
-   * Whether to separate rows.
-   *
-   * Equivalent to using the
-   * [`.boxed-list-separate`](style-classes.html#boxed-lists-cards) style class
-   * on a [class@Gtk.ListBox] instead of `.boxed-list`.
-   *
-   * Since: 1.6
-   */
-  props[PROP_SEPARATE_ROWS] =
-    g_param_spec_boolean ("separate-rows", NULL, NULL,
-                          FALSE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   gtk_widget_class_set_template_from_resource (widget_class,
@@ -404,11 +374,10 @@ adw_preferences_group_add (AdwPreferencesGroup *self,
 
   g_return_if_fail (ADW_IS_PREFERENCES_GROUP (self));
   g_return_if_fail (GTK_IS_WIDGET (child));
-  g_return_if_fail (gtk_widget_get_parent (child) == NULL);
 
   priv = adw_preferences_group_get_instance_private (self);
 
-  if (GTK_IS_LIST_BOX_ROW (child))
+  if (ADW_IS_PREFERENCES_ROW (child))
     gtk_list_box_append (priv->listbox, child);
   else
     gtk_box_append (priv->listbox_box, child);
@@ -444,33 +413,7 @@ adw_preferences_group_remove (AdwPreferencesGroup *self,
 }
 
 /**
- * adw_preferences_group_get_row:
- * @self: a preferences group
- * @index: a row index
- *
- * Gets the row at @index.
- *
- * Can return `NULL` if @index is larger than the number of rows in the group.
- *
- * Returns: (transfer none) (nullable): the row at @index
- *
- * Since: 1.8
- */
-GtkWidget *
-adw_preferences_group_get_row (AdwPreferencesGroup *self,
-                               guint                index)
-{
-  AdwPreferencesGroupPrivate *priv;
-
-  g_return_val_if_fail (ADW_IS_PREFERENCES_GROUP (self), NULL);
-
-  priv = adw_preferences_group_get_instance_private (self);
-
-  return GTK_WIDGET (gtk_list_box_get_row_at_index (priv->listbox, index));
-}
-
-/**
- * adw_preferences_group_get_title:
+ * adw_preferences_group_get_title: (attributes org.gtk.Method.get_property=title)
  * @self: a preferences group
  *
  * Gets the title of @self.
@@ -490,7 +433,7 @@ adw_preferences_group_get_title (AdwPreferencesGroup *self)
 }
 
 /**
- * adw_preferences_group_set_title:
+ * adw_preferences_group_set_title: (attributes org.gtk.Method.set_property=title)
  * @self: a preferences group
  * @title: the title
  *
@@ -517,7 +460,7 @@ adw_preferences_group_set_title (AdwPreferencesGroup *self,
 }
 
 /**
- * adw_preferences_group_get_description:
+ * adw_preferences_group_get_description: (attributes org.gtk.Method.get_property=description)
  * @self: a preferences group
  *
  * Gets the description of @self.
@@ -537,7 +480,7 @@ adw_preferences_group_get_description (AdwPreferencesGroup *self)
 }
 
 /**
- * adw_preferences_group_set_description:
+ * adw_preferences_group_set_description: (attributes org.gtk.Method.set_property=description)
  * @self: a preferences group
  * @description: (nullable): the description
  *
@@ -565,7 +508,7 @@ adw_preferences_group_set_description (AdwPreferencesGroup *self,
 
 /**
  * adw_preferences_group_get_header_suffix:
- * @self: a preferences group
+ * @self: a `AdwPreferencesGroup`
  *
  * Gets the suffix for @self's header.
  *
@@ -587,7 +530,7 @@ adw_preferences_group_get_header_suffix (AdwPreferencesGroup *self)
 
 /**
  * adw_preferences_group_set_header_suffix:
- * @self: a preferences group
+ * @self: a `AdwPreferencesGroup`
  * @suffix: (nullable): the suffix to set
  *
  * Sets the suffix for @self's header.
@@ -612,9 +555,6 @@ adw_preferences_group_set_header_suffix (AdwPreferencesGroup *self,
   if (suffix == priv->header_suffix)
     return;
 
-  if (suffix)
-    g_return_if_fail (gtk_widget_get_parent (suffix) == NULL);
-
   if (priv->header_suffix)
     gtk_box_remove (priv->header_box, priv->header_suffix);
 
@@ -624,102 +564,6 @@ adw_preferences_group_set_header_suffix (AdwPreferencesGroup *self,
     gtk_box_append (priv->header_box, priv->header_suffix);
 
   update_header_visibility (self);
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_HEADER_SUFFIX]);
-}
-
-/**
- * adw_preferences_group_get_separate_rows:
- * @self: a preferences group
- *
- * Gets whether @self's rows are separated.
- *
- * Returns: whether rows are separated
- *
- * Since: 1.6
- */
-gboolean
-adw_preferences_group_get_separate_rows (AdwPreferencesGroup *self)
-{
-  AdwPreferencesGroupPrivate *priv;
-
-  g_return_val_if_fail (ADW_IS_PREFERENCES_GROUP (self), FALSE);
-
-  priv = adw_preferences_group_get_instance_private (self);
-
-  return gtk_widget_has_css_class (GTK_WIDGET (priv->listbox), "boxed-list-separate");
-}
-
-/**
- * adw_preferences_group_set_separate_rows:
- * @self: a preferences group
- * @separate_rows: whether to separate rows
- *
- * Sets whether @self's rows are separated.
- *
- * Equivalent to using the
- * [`.boxed-list-separate`](style-classes.html#boxed-lists-cards) style class
- * on a [class@Gtk.ListBox] instead of `.boxed-list`.
- *
- * Since: 1.6
- */
-void
-adw_preferences_group_set_separate_rows (AdwPreferencesGroup *self,
-                                         gboolean             separate_rows)
-{
-  AdwPreferencesGroupPrivate *priv;
-
-  g_return_if_fail (ADW_IS_PREFERENCES_GROUP (self));
-
-  priv = adw_preferences_group_get_instance_private (self);
-
-  separate_rows = !!separate_rows;
-
-  if (separate_rows == adw_preferences_group_get_separate_rows (self))
-    return;
-
-  if (separate_rows) {
-    gtk_widget_add_css_class (GTK_WIDGET (priv->listbox), "boxed-list-separate");
-    gtk_widget_remove_css_class (GTK_WIDGET (priv->listbox), "boxed-list");
-  } else {
-    gtk_widget_add_css_class (GTK_WIDGET (priv->listbox), "boxed-list");
-    gtk_widget_remove_css_class (GTK_WIDGET (priv->listbox), "boxed-list-separate");
-  }
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SEPARATE_ROWS]);
-}
-
-/**
- * adw_preferences_group_bind_model:
- * @self: a preferences group
- * @model: (nullable): a list model to bind
- * @create_row_func: (nullable) (scope notified) (closure user_data) (destroy user_data_free_func):
- *   a function creating a row for each item, or `NULL` in case @model is `NULL`
- * @user_data: user data passed to @create_row_func
- * @user_data_free_func: function for freeing @user_data
- *
- * Binds @model to @self.
- *
- * See [method@Gtk.ListBox.bind_model].
- *
- * Since: 1.8
- */
-void
-adw_preferences_group_bind_model (AdwPreferencesGroup        *self,
-                                  GListModel                 *model,
-                                  GtkListBoxCreateWidgetFunc  create_row_func,
-                                  gpointer                    user_data,
-                                  GDestroyNotify              user_data_free_func)
-{
-  AdwPreferencesGroupPrivate *priv;
-
-  g_return_if_fail (ADW_IS_PREFERENCES_GROUP (self));
-  g_return_if_fail (model == NULL || G_IS_LIST_MODEL (model));
-  g_return_if_fail (model == NULL || create_row_func != NULL);
-
-  priv = adw_preferences_group_get_instance_private (self);
-
-  gtk_list_box_bind_model (priv->listbox, model, create_row_func, user_data, user_data_free_func);
 }
 
 /**
