@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * Author: Alice Mikhaylenko <alice.mikhaylenko@puri.sm>
+ * Author: Alexander Mikhaylenko <alexander.mikhaylenko@puri.sm>
  */
 
 #include "config.h"
@@ -11,7 +11,6 @@
 
 #include "adw-fading-label-private.h"
 #include "adw-gizmo-private.h"
-#include "adw-spinner-paintable.h"
 #include "adw-tab-view-private.h"
 #include "adw-timed-animation.h"
 
@@ -26,7 +25,9 @@ struct _AdwTabThumbnail
   GtkWidget *icon_title_box;
   GtkWidget *overlay;
   GtkPicture *picture;
-  GtkWidget *icon;
+  GtkWidget *icon_stack;
+  GtkImage *icon;
+  GtkSpinner *spinner;
   GtkImage *indicator_icon;
   GtkWidget *indicator_btn;
   GtkWidget *close_btn;
@@ -39,7 +40,6 @@ struct _AdwTabThumbnail
   AdwTabView *view;
   AdwTabPage *page;
   gboolean pinned;
-  gboolean loading;
 
   gboolean inverted;
 
@@ -91,30 +91,34 @@ update_tooltip (AdwTabThumbnail *self)
                                  adw_tab_page_get_title (page));
 }
 
+static void
+update_spinner (AdwTabThumbnail *self)
+{
+  gboolean loading = self->page && adw_tab_page_get_loading (self->page);
+  gboolean mapped = gtk_widget_get_mapped (GTK_WIDGET (self));
+
+  /* Don't use CPU when not needed */
+  gtk_spinner_set_spinning (self->spinner, loading && mapped);
+}
+
 static void
 update_icon (AdwTabThumbnail *self)
 {
   GIcon *gicon = adw_tab_page_get_icon (self->page);
   gboolean loading = adw_tab_page_get_loading (self->page);
+  const char *name = loading ? "spinner" : "icon";
 
-  if (loading && !self->loading) {
-    AdwSpinnerPaintable *paintable = adw_spinner_paintable_new (self->icon);
-
-    gtk_image_set_from_paintable (GTK_IMAGE (self->icon), GDK_PAINTABLE (paintable));
-
-    g_object_unref (paintable);
-  } else if (!loading) {
-    gtk_image_set_from_gicon (GTK_IMAGE (self->icon), gicon);
-  }
-
-  self->loading = loading;
-  gtk_widget_set_visible (self->icon, (gicon != NULL || loading));
+  gtk_image_set_from_gicon (self->icon, gicon);
+  gtk_widget_set_visible (self->icon_stack,
+                          (gicon != NULL || loading));
+  gtk_stack_set_visible_child_name (GTK_STACK (self->icon_stack), name);
 }
 
 static void
 update_loading (AdwTabThumbnail *self)
 {
   update_icon (self);
+  update_spinner (self);
   set_style_class (GTK_WIDGET (self), "loading",
                    adw_tab_page_get_loading (self->page));
 }
@@ -140,24 +144,12 @@ update_indicator (AdwTabThumbnail *self)
   set_style_class (GTK_WIDGET (self), "indicator", indicator != NULL);
 }
 
-static void
-update_has_popup (AdwTabThumbnail *self)
-{
-  if (adw_tab_view_get_menu_model (self->view)) {
-    gtk_accessible_update_property (GTK_ACCESSIBLE (self),
-                                    GTK_ACCESSIBLE_PROPERTY_HAS_POPUP, TRUE,
-                                    -1);
-  } else {
-    gtk_accessible_reset_property (GTK_ACCESSIBLE (self),
-                                   GTK_ACCESSIBLE_PROPERTY_HAS_POPUP);
-  }
-}
-
-static void
+static gboolean
 close_idle_cb (AdwTabThumbnail *self)
 {
   adw_tab_view_close_page (self->view, self->page);
-  g_object_unref (self);
+
+  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -169,14 +161,15 @@ close_clicked_cb (AdwTabThumbnail *self)
   /* When animations are disabled, we don't want to immediately remove the
    * whole tab mid-click. Instead, defer it until the click has happened.
    */
-  g_idle_add_once ((GSourceOnceFunc) close_idle_cb, g_object_ref (self));
+  g_idle_add ((GSourceFunc) close_idle_cb, self);
 }
 
-static void
+static gboolean
 unpin_idle_cb (AdwTabThumbnail *self)
 {
   adw_tab_view_set_page_pinned (self->view, self->page, FALSE);
-  g_object_unref (self);
+
+  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -188,7 +181,7 @@ unpin_clicked_cb (AdwTabThumbnail *self)
   /* When animations are disabled, we don't want to immediately unpin the
    * whole tab mid-click. Instead, defer it until the click has happened.
    */
-  g_idle_add_once ((GSourceOnceFunc) unpin_idle_cb, g_object_ref (self));
+  g_idle_add ((GSourceFunc) unpin_idle_cb, self);
 }
 
 static void
@@ -200,6 +193,41 @@ indicator_clicked_cb (AdwTabThumbnail *self)
   g_signal_emit_by_name (self->view, "indicator-activated", self->page);
 }
 
+static gboolean
+drop_cb (AdwTabThumbnail *self,
+         GValue          *value)
+{
+  gboolean ret = GDK_EVENT_PROPAGATE;
+
+  g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_DROP], 0, value, &ret);
+
+  return ret;
+}
+
+static GdkDragAction
+extra_drag_enter_cb (AdwTabThumbnail *self)
+{
+  const GValue *value = gtk_drop_target_get_value (self->drop_target);
+
+  g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_VALUE], 0, value, &self->preferred_action);
+
+  return self->preferred_action;
+}
+
+static GdkDragAction
+extra_drag_motion_cb (AdwTabThumbnail *self)
+{
+  return self->preferred_action;
+}
+
+static void
+extra_drag_notify_value_cb (AdwTabThumbnail *self)
+{
+  const GValue *value = gtk_drop_target_get_value (self->drop_target);
+
+  g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_VALUE], 0, value, &self->preferred_action);
+}
+
 static GdkDragAction
 make_action_unique (GdkDragAction actions)
 {
@@ -215,45 +243,6 @@ make_action_unique (GdkDragAction actions)
   return 0;
 }
 
-static gboolean
-drop_cb (AdwTabThumbnail *self,
-         GValue          *value)
-{
-  gboolean ret = GDK_EVENT_PROPAGATE;
-  GdkDrop *drop = gtk_drop_target_get_current_drop (self->drop_target);
-  GdkDragAction preferred_action = gdk_drop_get_actions (drop);
-
-  g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_DROP], 0, value, preferred_action, &ret);
-
-  return ret;
-}
-
-static GdkDragAction
-extra_drag_enter_cb (AdwTabThumbnail *self)
-{
-  const GValue *value = gtk_drop_target_get_value (self->drop_target);
-
-  g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_VALUE], 0, value, &self->preferred_action);
-  self->preferred_action = make_action_unique (self->preferred_action);
-
-  return self->preferred_action;
-}
-
-static GdkDragAction
-extra_drag_motion_cb (AdwTabThumbnail *self)
-{
-  return self->preferred_action;
-}
-
-static void
-extra_drag_notify_value_cb (AdwTabThumbnail *self)
-{
-  const GValue *value = gtk_drop_target_get_value (self->drop_target);
-
-  g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_VALUE], 0, value, &self->preferred_action);
-  self->preferred_action = make_action_unique (self->preferred_action);
-}
-
 static void
 fade_animation_value_cb (double           value,
                          AdwTabThumbnail *self)
@@ -269,6 +258,26 @@ fade_animation_value_cb (double           value,
   gtk_widget_set_opacity (self->needs_attention_revealer, value);
 }
 
+static void
+adw_tab_thumbnail_map (GtkWidget *widget)
+{
+  AdwTabThumbnail *self = ADW_TAB_THUMBNAIL (widget);
+
+  GTK_WIDGET_CLASS (adw_tab_thumbnail_parent_class)->map (widget);
+
+  update_spinner (self);
+}
+
+static void
+adw_tab_thumbnail_unmap (GtkWidget *widget)
+{
+  AdwTabThumbnail *self = ADW_TAB_THUMBNAIL (widget);
+
+  GTK_WIDGET_CLASS (adw_tab_thumbnail_parent_class)->unmap (widget);
+
+  update_spinner (self);
+}
+
 static void
 measure_pinned_tab (AdwGizmo       *gizmo,
                     GtkOrientation  orientation,
@@ -392,12 +401,6 @@ adw_tab_thumbnail_constructed (GObject *object)
 
     gtk_widget_set_visible (GTK_WIDGET (self->picture), FALSE);
   }
-
-  g_signal_connect_object (self->view, "notify::menu-model",
-                           G_CALLBACK (update_has_popup), self,
-                           G_CONNECT_SWAPPED);
-
-  update_has_popup (self);
 }
 
 static void
@@ -485,6 +488,9 @@ adw_tab_thumbnail_class_init (AdwTabThumbnailClass *klass)
   object_class->get_property = adw_tab_thumbnail_get_property;
   object_class->set_property = adw_tab_thumbnail_set_property;
 
+  widget_class->map = adw_tab_thumbnail_map;
+  widget_class->unmap = adw_tab_thumbnail_unmap;
+
   props[PROP_VIEW] =
     g_param_spec_object ("view", NULL, NULL,
                          ADW_TYPE_TAB_VIEW,
@@ -515,9 +521,8 @@ adw_tab_thumbnail_class_init (AdwTabThumbnailClass *klass)
                   g_signal_accumulator_first_wins,
                   NULL, NULL,
                   G_TYPE_BOOLEAN,
-                  2,
-                  G_TYPE_VALUE,
-                  GDK_TYPE_DRAG_ACTION);
+                  1,
+                  G_TYPE_VALUE);
 
   signals[SIGNAL_EXTRA_DRAG_VALUE] =
     g_signal_new ("extra-drag-value",
@@ -536,7 +541,9 @@ adw_tab_thumbnail_class_init (AdwTabThumbnailClass *klass)
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, overlay);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, icon_title_box);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, picture);
+  gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, icon_stack);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, icon);
+  gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, spinner);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, indicator_icon);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, indicator_btn);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, close_btn);
@@ -571,8 +578,6 @@ adw_tab_thumbnail_init (AdwTabThumbnail *self)
                                                   0, 1,
                                                   FADE_TRANSITION_DURATION,
                                                   target);
-
-  adw_timed_animation_set_easing (ADW_TIMED_ANIMATION (self->fade_animation), ADW_EASE);
 }
 
 AdwTabThumbnail *
@@ -620,6 +625,7 @@ adw_tab_thumbnail_set_page (AdwTabThumbnail *self,
     gtk_picture_set_paintable (GTK_PICTURE (self->picture), paintable);
 
     update_tooltip (self);
+    update_spinner (self);
     update_icon (self);
     update_indicator (self);
     update_loading (self);
@@ -724,8 +730,6 @@ adw_tab_thumbnail_fade_out (AdwTabThumbnail *self)
   g_return_if_fail (ADW_IS_TAB_THUMBNAIL (self));
 
   adw_animation_reset (self->fade_animation);
-
-  gtk_widget_set_opacity (self->overlay, 0);
 }
 
 void
@@ -733,7 +737,5 @@ adw_tab_thumbnail_fade_in (AdwTabThumbnail *self)
 {
   g_return_if_fail (ADW_IS_TAB_THUMBNAIL (self));
 
-  gtk_widget_set_opacity (self->overlay, 1);
-
   adw_animation_play (self->fade_animation);
 }
